@@ -20,10 +20,8 @@
 #include <config.h>
 #endif
 
-#include <exo/exo.h>
 #include <gtk/gtk.h>
 #include <libxfce4ui/libxfce4ui.h>
-#include <libxfce4util/libxfce4util.h>
 #include <X11/Xlib.h>
 #include <X11/extensions/XTest.h>
 #include <X11/keysym.h>
@@ -162,7 +160,8 @@ cb_set_clipboard (GtkMenuItem *mi, const ClipmanHistoryItem *item)
       break;
 
     default:
-      g_assert_not_reached ();
+      DBG("Ignoring unknown history type %d", item->type);
+      return;
     }
 
   {
@@ -236,13 +235,11 @@ cb_clear_history (ClipmanMenu *menu)
 
 
       {
-#if GTK_CHECK_VERSION(2, 22, 0)
         GtkWidget *content_area = gtk_message_dialog_get_message_area (GTK_MESSAGE_DIALOG (dialog));
-#else
-        GtkWidget *content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
-#endif
         GtkWidget *checkbox = gtk_check_button_new_with_label (_("Don't ask again"));
-        exo_binding_new (G_OBJECT (checkbox), "active", G_OBJECT (menu), "never-confirm-history-clear");
+        g_object_bind_property(G_OBJECT (checkbox), "active",
+                               G_OBJECT (menu), "never-confirm-history-clear",
+                               G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
         gtk_widget_show (checkbox);
         gtk_container_add (GTK_CONTAINER (content_area), checkbox);
 
@@ -261,9 +258,11 @@ cb_clear_history (ClipmanMenu *menu)
   clipman_history_clear (menu->priv->history);
 
   clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
+  gtk_clipboard_set_text (clipboard, "", 1);
   gtk_clipboard_clear (clipboard);
 
   clipboard = gtk_clipboard_get (GDK_SELECTION_PRIMARY);
+  gtk_clipboard_set_text (clipboard, "", 1);
   gtk_clipboard_clear (clipboard);
 }
 
@@ -303,17 +302,22 @@ _clipman_menu_update_list (ClipmanMenu *menu)
       switch (item->type)
         {
         case CLIPMAN_HISTORY_TYPE_TEXT:
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
           mi = gtk_image_menu_item_new_with_label (item->preview.text);
+G_GNUC_END_IGNORE_DEPRECATIONS
           break;
 
         case CLIPMAN_HISTORY_TYPE_IMAGE:
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
           mi = gtk_image_menu_item_new ();
           image = gtk_image_new_from_pixbuf (item->preview.image);
           gtk_container_add (GTK_CONTAINER (mi), image);
+G_GNUC_END_IGNORE_DEPRECATIONS
           break;
 
         default:
-          g_assert_not_reached ();
+          DBG("Ignoring unknown history type %d", item->type);
+          continue;
         }
 
       g_signal_connect (mi, "activate", G_CALLBACK (cb_set_clipboard), item);
@@ -321,8 +325,10 @@ _clipman_menu_update_list (ClipmanMenu *menu)
 
       if (item == item_to_restore)
         {
-          image = gtk_image_new_from_stock (GTK_STOCK_GO_FORWARD, GTK_ICON_SIZE_MENU);
+          image = gtk_image_new_from_icon_name ("go-next-symbolic", GTK_ICON_SIZE_MENU);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
           gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), image);
+G_GNUC_END_IGNORE_DEPRECATIONS
         }
 
       menu->priv->list = g_slist_prepend (menu->priv->list, mi);
@@ -341,7 +347,9 @@ _clipman_menu_update_list (ClipmanMenu *menu)
 
       if ((pixbuf = clipman_menu_qrcode (item_to_restore->content.text)) != NULL)
         {
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
           mi = gtk_image_menu_item_new ();
+G_GNUC_END_IGNORE_DEPRECATIONS
           gtk_container_add (GTK_CONTAINER (mi), gtk_image_new_from_pixbuf (pixbuf));
           g_signal_connect (mi, "activate", G_CALLBACK (cb_set_qrcode), pixbuf);
           menu->priv->list = g_slist_prepend (menu->priv->list, mi);
@@ -449,6 +457,7 @@ static void
 clipman_menu_init (ClipmanMenu *menu)
 {
   GtkWidget *mi;
+  GtkWidget *image;
 
   menu->priv = GET_PRIVATE (menu);
 
@@ -462,7 +471,11 @@ clipman_menu_init (ClipmanMenu *menu)
   mi = gtk_separator_menu_item_new ();
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
 
-  menu->priv->mi_clear_history = mi = gtk_image_menu_item_new_from_stock (GTK_STOCK_CLEAR, NULL);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+  menu->priv->mi_clear_history = mi = gtk_image_menu_item_new_with_mnemonic (_("_Clear history"));
+  image = gtk_image_new_from_icon_name ("edit-clear-symbolic", GTK_ICON_SIZE_MENU);
+  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu->priv->mi_clear_history), image);
+G_GNUC_END_IGNORE_DEPRECATIONS
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
   g_signal_connect_swapped (mi, "activate", G_CALLBACK (cb_clear_history), menu);
 
@@ -546,7 +559,7 @@ clipman_menu_qrcode (char *text)
 	int i, j, k, rowstride, channels;
 	guchar *pixel;
 	unsigned char *data;
-	
+
 	qrcode = QRcode_encodeString8bit(text, 0, QR_ECLEVEL_L);
 
 	if (qrcode == NULL)
@@ -572,7 +585,7 @@ clipman_menu_qrcode (char *text)
 
 	QRcode_free(qrcode);
 	g_object_unref(pixbuf);
-	
+
 	return pixbuf_scaled;
 }
 #endif
