@@ -43,8 +43,9 @@
  */
 
 static gboolean         my_plugin_set_popup_selection   (MyPlugin *plugin);
-static gboolean         cb_popup_message_received       (MyPlugin *plugin,
-                                                         GdkEventClient *ev);
+static GdkFilterReturn  event_filter_popup_menu         (GdkXEvent *xevent,
+                                                         GdkEvent *event,
+                                                         MyPlugin *plugin);
 static gboolean         xfce_popup_grab_available       (GdkWindow *win,
                                                          guint32 timestamp);
 
@@ -56,7 +57,7 @@ clipboard_manager_ownership_exists (void)
   Display *display;
   Atom atom;
 
-  display = GDK_DISPLAY ();
+  display = gdk_x11_get_default_xdisplay ();
   atom = XInternAtom (display, "CLIPBOARD_MANAGER", FALSE);
   return XGetSelectionOwner (display, atom);
 }
@@ -132,7 +133,6 @@ plugin_register (void)
                             G_CALLBACK (plugin_save), plugin);
   g_signal_connect_swapped (plugin->history, "clear",
                             G_CALLBACK (plugin_save), plugin);
-
   /* Set the selection for the popup command */
   my_plugin_set_popup_selection (plugin);
 
@@ -280,6 +280,7 @@ plugin_free (MyPlugin *plugin)
       gsd_clipboard_manager_stop (plugin->daemon);
       g_object_unref (plugin->daemon);
     }
+  gdk_window_remove_filter (gtk_widget_get_window(plugin->menu), (GdkFilterFunc) event_filter_popup_menu, plugin);
   gtk_widget_destroy (plugin->menu);
   g_object_unref (plugin->channel);
   g_object_unref (plugin->actions);
@@ -296,26 +297,6 @@ plugin_free (MyPlugin *plugin)
   xfconf_shutdown ();
 }
 
-static void
-cb_about_dialog_url_hook (GtkAboutDialog *dialog,
-                          const gchar *uri,
-                          gpointer user_data)
-{
-  gchar *command = NULL;
-
-  if (!gtk_show_uri (NULL, uri, GDK_CURRENT_TIME, NULL))
-    {
-      command = g_strdup_printf ("exo-open --launch %s", uri);
-      if (!g_spawn_command_line_async (command, NULL))
-        {
-          g_free (command);
-          command = g_strdup_printf ("firefox %s", uri);
-          g_spawn_command_line_async (command, NULL);
-        }
-      g_free (command);
-    }
-}
-
 void
 plugin_about (MyPlugin *plugin)
 {
@@ -334,7 +315,6 @@ plugin_about (MyPlugin *plugin)
     "the Free Software Foundation; either version 2 of the License, or\n"
     "(at your option) any later version.\n";
 
-  gtk_about_dialog_set_url_hook (cb_about_dialog_url_hook, NULL, NULL);
   gtk_show_about_dialog (NULL,
                          "program-name", _("Clipman"),
                          "logo-icon-name", "xfce4-clipman-plugin",
@@ -373,16 +353,27 @@ plugin_popup_menu (MyPlugin *plugin)
 {
 #ifdef PANEL_PLUGIN
   gtk_menu_set_screen (GTK_MENU (plugin->menu), gtk_widget_get_screen (plugin->button));
+
+  if(!gtk_widget_has_grab(plugin->menu))
+  {
+    gtk_grab_add(plugin->menu);
+  }
+
   gtk_menu_popup (GTK_MENU (plugin->menu), NULL, NULL,
                   plugin->menu_position_func, plugin,
                   0, gtk_get_current_event_time ());
-  if (gtk_grab_get_current () == plugin->menu)
-    {
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (plugin->button), TRUE);
-      xfce_panel_plugin_register_menu (plugin->panel_plugin, GTK_MENU (plugin->menu));
-    }
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (plugin->button), TRUE);
+  xfce_panel_plugin_register_menu (plugin->panel_plugin, GTK_MENU (plugin->menu));
 #elif defined (STATUS_ICON)
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   gtk_menu_set_screen (GTK_MENU (plugin->menu), gtk_status_icon_get_screen (plugin->status_icon));
+G_GNUC_END_IGNORE_DEPRECATIONS
+
+  if(!gtk_widget_has_grab(plugin->menu))
+  {
+    gtk_grab_add(plugin->menu);
+  }
+
   gtk_menu_popup (GTK_MENU (plugin->menu), NULL, NULL,
                   plugin->menu_position_func, plugin->status_icon,
                   0, gtk_get_current_event_time ());
@@ -401,56 +392,74 @@ my_plugin_set_popup_selection (MyPlugin *plugin)
   Atom                selection_atom;
   GtkWidget          *win;
   Window              id;
+  Display            *display;
+  GdkWindow          *window;
 
   win = gtk_invisible_new ();
   gtk_widget_realize (win);
   id = GDK_WINDOW_XID (gtk_widget_get_window (win));
+  display = gdk_x11_get_default_xdisplay ();
 
   gscreen = gtk_widget_get_screen (win);
   selection_name = g_strdup_printf (XFCE_CLIPMAN_SELECTION"%d",
                                     gdk_screen_get_number (gscreen));
-  selection_atom = XInternAtom (GDK_DISPLAY(), selection_name, FALSE);
+  selection_atom = XInternAtom (display, selection_name, FALSE);
+  g_free(selection_name);
 
-  if (XGetSelectionOwner (GDK_DISPLAY(), selection_atom))
+  if (XGetSelectionOwner (display, selection_atom))
     {
       gtk_widget_destroy (win);
       return FALSE;
     }
 
-  XSelectInput (GDK_DISPLAY(), id, PropertyChangeMask);
-  XSetSelectionOwner (GDK_DISPLAY(), selection_atom, id, GDK_CURRENT_TIME);
+  XSelectInput (display, id, PropertyChangeMask);
+  XSetSelectionOwner (display, selection_atom, id, GDK_CURRENT_TIME);
 
-  g_signal_connect_swapped (win, "client-event",
-                            G_CALLBACK (cb_popup_message_received), plugin);
+  window = gtk_widget_get_window (win);
+  gdk_window_add_filter (window, (GdkFilterFunc) event_filter_popup_menu, plugin);
 
   return TRUE;
 }
 
-static gboolean
-cb_popup_message_received (MyPlugin *plugin,
-                           GdkEventClient *ev)
+static GdkFilterReturn
+event_filter_popup_menu (GdkXEvent *xevent, GdkEvent *event, MyPlugin *plugin)
 {
-  {
+    XClientMessageEvent *evt;
+    GdkScreen *screen;
+    GdkWindow *root;
+    Atom message_type;
+    evt = (XClientMessageEvent *)xevent;
+
+    if (((XEvent *)xevent)->type != ClientMessage)
+      return GDK_FILTER_CONTINUE;
+
+    message_type = XInternAtom (gdk_x11_get_default_xdisplay (), "STRING", FALSE);
+    if (evt->message_type != message_type)
+      return GDK_FILTER_CONTINUE;
+
     /* Copy workaround from xfdesktop to handle the awkward case where binding
      * a keyboard shortcut to the popup command doesn't always work out... */
 #ifdef PANEL_PLUGIN
-    GdkScreen *screen = gtk_widget_get_screen (GTK_WIDGET (plugin->button));
+    screen = gtk_widget_get_screen (GTK_WIDGET (plugin->button));
 #elif defined (STATUS_ICON)
-    GdkScreen *screen = gtk_status_icon_get_screen (plugin->status_icon);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+    screen = gtk_status_icon_get_screen (plugin->status_icon);
+G_GNUC_END_IGNORE_DEPRECATIONS
 #endif
-    GdkWindow *root = gdk_screen_get_root_window (screen);
+    root = gdk_screen_get_root_window (screen);
+
     if (!xfce_popup_grab_available (root, GDK_CURRENT_TIME))
       {
         g_critical ("Unable to get keyboard/mouse grab.");
         return FALSE;
       }
-  }
 
-  if (G_LIKELY (ev->data_format == 8 && *(ev->data.b) != '\0'))
+  if (G_LIKELY (evt->format == 8) && (*(evt->data.b) != '\0'))
     {
-      if (!g_ascii_strcasecmp (XFCE_CLIPMAN_MESSAGE, ev->data.b))
+
+      if (!g_ascii_strcasecmp (XFCE_CLIPMAN_MESSAGE, evt->data.b))
         {
-          DBG ("Message received: %s", ev->data.b);
+          DBG ("Message received: %s", evt->data.b);
 
           if (xfconf_channel_get_bool (plugin->channel, "/tweaks/popup-at-pointer", FALSE))
             {
@@ -463,12 +472,25 @@ cb_popup_message_received (MyPlugin *plugin,
             }
 
           return TRUE;
+        } else if (!g_ascii_strcasecmp (XFCE_CLIPMAN_ACTION_MESSAGE, evt->data.b))
+        {
+             clipman_collector_show_actions();
         }
     }
 
   return FALSE;
 }
 
+#if GTK_CHECK_VERSION (3, 20, 0)
+static void
+make_window_visible (GdkSeat *seat,
+                     GdkWindow *window,
+                     gpointer user_data)
+{
+  gdk_window_show (window);
+}
+#endif
+
 /* Code taken from xfwm4/src/menu.c:grab_available().  This should fix the case
  * where binding 'xfdesktop -menu' to a keyboard shortcut sometimes works and
  * sometimes doesn't.  Credit for this one goes to Olivier.
@@ -476,44 +498,40 @@ cb_popup_message_received (MyPlugin *plugin,
 static gboolean
 xfce_popup_grab_available (GdkWindow *win, guint32 timestamp)
 {
-    GdkEventMask mask =
-        GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
-        GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK |
-        GDK_POINTER_MOTION_MASK;
-    GdkGrabStatus g1;
-    GdkGrabStatus g2;
-    gboolean grab_failed = FALSE;
+    GdkDisplay* display = gdk_window_get_display(win);
+#if GTK_CHECK_VERSION (3, 20, 0)
+    GdkSeat *seat = gdk_display_get_default_seat (display);
+#endif
+    GdkGrabStatus g = GDK_GRAB_ALREADY_GRABBED;
+    gboolean grab_failed = TRUE;
     gint i = 0;
 
     TRACE ("entering grab_available");
 
-    g1 = gdk_pointer_grab (win, TRUE, mask, NULL, NULL, timestamp);
-    g2 = gdk_keyboard_grab (win, TRUE, timestamp);
-
-    while ((i++ < 2500) && (grab_failed = ((g1 != GDK_GRAB_SUCCESS)
-                || (g2 != GDK_GRAB_SUCCESS))))
+    /* With a keyboard grab elsewhere, we have to wait on that to clear.
+     * So try up to 2500 times and only keep trying when the failure is
+     * already grabbed, any other failure mode will never succeed.
+     */
+    while ((i++ < 2500) && grab_failed && g == GDK_GRAB_ALREADY_GRABBED)
     {
-        TRACE ("grab not available yet, waiting... (%i)", i);
-        g_usleep (100);
-        if (g1 != GDK_GRAB_SUCCESS)
-        {
-            g1 = gdk_pointer_grab (win, TRUE, mask, NULL, NULL, timestamp);
-        }
-        if (g2 != GDK_GRAB_SUCCESS)
-        {
-            g2 = gdk_keyboard_grab (win, TRUE, timestamp);
-        }
-    }
-
-    if (g1 == GDK_GRAB_SUCCESS)
-    {
-        gdk_pointer_ungrab (timestamp);
-    }
-    if (g2 == GDK_GRAB_SUCCESS)
-    {
-        gdk_keyboard_ungrab (timestamp);
+#if GTK_CHECK_VERSION (3, 20, 0)
+      g = gdk_seat_grab(seat, win, GDK_SEAT_CAPABILITY_KEYBOARD, TRUE, NULL, NULL, make_window_visible, NULL);
+      if (g == GDK_GRAB_SUCCESS)
+      {
+          gdk_seat_ungrab (seat);
+          grab_failed = FALSE;
+      }
+#else
+      G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+      g = gdk_keyboard_grab (win, TRUE, timestamp);
+      if (g == GDK_GRAB_SUCCESS)
+      {
+          gdk_keyboard_ungrab(timestamp);
+          grab_failed = FALSE;
+      }
+      G_GNUC_END_IGNORE_DEPRECATIONS
+#endif
     }
 
     return (!grab_failed);
 }
-
